//
// serial.c
//
// serial functions for cygwin
// designed for the pcmflash and pcmread programs
//

#include <stdio.h>          /* Standard input/output definitions */
#include <string.h>         /* String function definitions */
#include <unistd.h>         /* UNIX standard function definitions */
#include <fcntl.h>          /* File control definitions */
#include <errno.h>          /* Error number definitions */
#include <termios.h>        /* POSIX terminal control definitions */
#include <sys/signal.h>

#define DEFAULT_TIMEOUT 5
#define FALSE 0
#define TRUE  !FALSE

int fd;
struct termios options, oldterm;

int init_com(int baud) 
{
  fd = open("/dev/ttyS0", O_RDWR | O_NOCTTY | O_NDELAY | O_BINARY);
  if (fd < 0){
      perror("/dev/ttyS0");
      return(fd);
  }    

  fcntl(fd, F_SETFL, FNDELAY);

  tcgetattr(fd, &oldterm);
  bzero(&options, sizeof(options));
  cfsetispeed(&options, baud);
  cfsetospeed(&options, baud);

  options.c_cflag |= (CLOCAL | CREAD); /* enable */
  options.c_cflag &= ~ CRTSCTS;        // no hardware flow control

  options.c_cflag &= ~PARENB;          /* 8N1 */
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS8;
  options.c_lflag = 0 ;                           // RAW INPUT MODE
  options.c_iflag = IGNPAR;
  
  options.c_oflag = 0 ;                          // raw output mode
  
  /* set all of the options */
  tcflush(fd, TCIOFLUSH);
  tcsetattr(fd, TCSANOW, &options);
  printf("Opened port %x\n",fd);
  return (fd);
}


int receive(char* buf, int length)
{
   int cnt = 0, n;
   int bytes;
   int timeout = DEFAULT_TIMEOUT;
   do
   {
           n = read(fd, &buf[cnt], length-cnt);
           if (n > 0){
               cnt += n;
               timeout = DEFAULT_TIMEOUT;
           }    
       else{
           --timeout;
           usleep(100);
       }
     }
   while((cnt<length) && timeout);
//   printf("%d characters received...\n",cnt);
   return cnt;
}

void transmit(char* buf, int length)
{
//   printf("transmitting...\n");
   int cnt = 0, n;
   int timeout = DEFAULT_TIMEOUT;
   buf[length] = 0;
   do
   {
      n = write(fd, &buf[cnt], length-cnt);
      if(n > 0)
      {
         cnt += n;
         timeout = DEFAULT_TIMEOUT;
      }
      else
      {
         --timeout;
         usleep(100);
      }
   }
   while(cnt<length && timeout);
//   printf("$%x characters transmitted\n",cnt);
   return;
}        

void restore(void)
{
    tcsetattr(fd, TCSAFLUSH, &oldterm);
    close(fd);
}    


